@@ -2,7 +2,7 @@
 #define _SHAPEFILE_H_INCLUDED
 
 /******************************************************************************
- * $Id: shapefil.h 2280 2006-01-16 10:42:58Z strk $
+ * $Id: shapefil.h 2786 2008-05-27 17:14:43Z pramsey $
  *
  * Project:  Shapelib
  * Purpose:  Primary include file for Shapelib.
@@ -456,6 +456,10 @@ const char SHPAPI_CALL1(*)
 int     SHPAPI_CALL
       DBFIsAttributeNULL( DBFHandle hDBF, int iShape, int iField );
 
+int SHPAPI_CALL DBFReadSetup(DBFHandle psDBF, int hEntity);
+
+int SHPAPI_CALL DBFReadDeleted(DBFHandle psDBF, int hEntity);
+
 int SHPAPI_CALL
       DBFWriteIntegerAttribute( DBFHandle hDBF, int iShape, int iField, 
                                 int nFieldValue );
