@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: shp2pgsql.c 2764 2008-04-12 16:44:55Z pramsey $
+ * $Id: shp2pgsql.c 2782 2008-05-27 02:59:06Z pramsey $
  *
  * PostGIS - Spatial Types for PostgreSQL
  * http://postgis.refractions.net
@@ -139,7 +139,7 @@ static void print_wkb_int(int val);
 static void print_wkb_double(double val);
 
 static char rcsid[] =
-  "$Id: shp2pgsql.c 2764 2008-04-12 16:44:55Z pramsey $";
+  "$Id: shp2pgsql.c 2782 2008-05-27 02:59:06Z pramsey $";
 
 void *safe_malloc(size_t size)
 {
@@ -673,6 +673,12 @@ LoadData(void)
 		trans++;
 		/* transaction stuff done */
 
+    /*skip the record if it has been deleted*/
+    if(readshape != 1 && DBFReadDeleted(hDBFHandle, j)) {
+      continue; 
+    }
+
+
 		/*open the next object */
 		if (readshape == 1)
 		{
