@@ -19,6 +19,8 @@
 #define FP_MIN(A, B) ((A < B) ? A : B)
 #define FP_LT(A, B) ((A + PGIS_EPSILON) < B)
 #define FP_LTEQ(A, B) ((A - PGIS_EPSILON) <= B)
+#define FP_GT(A, B) ((A - PGIS_EPSILON) > B)
+#define FP_GTEQ(A, B) ((A + PGIS_EPSILON) >= B)
 #define FP_CONTAINS_TOP(A, X, B) (FP_LT(A, X) && FP_LTEQ(X, B))
 #define FP_CONTAINS_BOTTOM(A, X, B) (FP_LTEQ(A, X) && FP_LT(X, B))
 #define FP_CONTAINS_INCL(A, X, B) (FP_LTEQ(A, X) && FP_LTEQ(X, B))
@@ -340,6 +342,9 @@ extern void lwgeom_addBBOX(LWGEOM *lwgeom);
 
 extern void lwgeom_dropSRID(LWGEOM *lwgeom);
 
+/* Determine whether a LWGEOM can contain sub-geometries or not */
+extern int lwgeom_contains_subgeoms(int type);
+
 /******************************************************************/
 
 /*
@@ -943,6 +948,7 @@ extern double distance2d_poly_poly(LWPOLY *poly1, LWPOLY *poly2);
 extern double distance2d_line_poly(LWLINE *line, LWPOLY *poly);
 extern int azimuth_pt_pt(POINT2D *p1, POINT2D *p2, double *ret);
 extern double lwgeom_mindistance2d_recursive(uchar *lw1, uchar *lw2);
+extern double lwgeom_mindistance2d_recursive_tolerance(uchar *lw1, uchar *lw2, double tolerance);
 extern int lwgeom_pt_inside_circle(POINT2D *p, double cx, double cy, double rad);
 extern int32 lwgeom_npoints(uchar *serialized);
 extern char ptarray_isccw(const POINTARRAY *pa);
@@ -1194,6 +1200,7 @@ extern int lwcurve_compute_box2d_p(LWCURVE *curve, BOX2DFLOAT4 *box);
 extern BOX3D *lwcurve_compute_box3d(LWCURVE *curve);
 extern void pfree_curve(LWCURVE  *curve);
 LWCURVE *lwcurve_clone(const LWCURVE *curve);
+LWCURVE *lwgeom_getcurve_inspected(LWGEOM_INSPECTED *inspected, int geom_number);
 
 /******************************************************************
  * LWMULTIx and LWCOLLECTION functions
