@@ -34,7 +34,7 @@ lwcurve_construct(int SRID, BOX2DFLOAT4 *bbox, POINTARRAY *points)
 {
         LWCURVE *result;
         
-	/*
+    /*
          * The first arc requires three points.  Each additional
          * arc requires two more points.  Thus the minimum point count
          * is three, and the count must be odd.
@@ -272,10 +272,10 @@ lwcircle_compute_box3d(POINT4D *p1, POINT4D *p2, POINT4D *p3)
 {
         double x1, x2, y1, y2, z1, z2;
         double angle, radius, sweep;
-        /*
-        double top, left;
-        */
+	// angles from center
         double a1, a2, a3;
+        // angles from center once a1 is rotated to zero
+	double r2, r3;
         double xe = 0.0, ye = 0.0;
         POINT4D *center;
         int i;
@@ -285,18 +285,16 @@ lwcircle_compute_box3d(POINT4D *p1, POINT4D *p2, POINT4D *p3)
         lwnotice("lwcircle_compute_box3d called.");
 #endif
 
+        /*
+         * Determine where the center of the circle lives, and the radius.
+         */
         center = lwalloc(sizeof(POINT4D));
         radius = lwcircle_center(p1, p2, p3, &center);
         if(radius < 0.0) return NULL;
 
-        /*
-        top = center->y + radius;
-        left = center->x - radius;
-
 #ifdef PGIS_DEBUG_CALLS
-        lwnotice("lwcircle_compute_box3d: top=%.16f, left=%.16f", top, left);
+        lwnotice("lwcircle_compute_box3d: center (%.16f, %.16f)", center->x, center->y);
 #endif
-        */
 
         x1 = MAXFLOAT;
         x2 = -1 * MAXFLOAT;
@@ -307,29 +305,54 @@ lwcircle_compute_box3d(POINT4D *p1, POINT4D *p2, POINT4D *p3)
         a2 = atan2(p2->y - center->y, p2->x - center->x);
         a3 = atan2(p3->y - center->y, p3->x - center->x);
 
-        /* Determine sweep angle */
-        if(a1 > a2 && a2 > a3) 
+        // Rotate a2 and a3 such that a1 = 0
+        r2 = a2 - a1;
+        r3 = a3 - a1;
+
+        /*
+         * There are six cases here I'm interested in
+         * Clockwise:
+         *   1. a1-a2 < 180 == r2 < 0 && (r3 > 0 || r3 < r2)
+         *   2. a1-a2 > 180 == r2 > 0 && (r3 > 0 && r3 < r2)
+         *   3. a1-a2 > 180 == r2 > 0 && (r3 > r2 || r3 < 0)
+         * Counter-clockwise:
+         *   4. a1-a2 < 180 == r2 > 0 && (r3 < 0 || r3 > r2)
+         *   5. a1-a2 > 180 == r2 < 0 && (r3 < 0 && r3 > r2)
+         *   6. a1-a2 > 180 == r2 < 0 && (r3 < r2 || r3 > 0)
+         * 3 and 6 are invalid cases where a3 is the midpoint.
+         * BBOX is fundamental, so these cannot error out and will instead
+         * calculate the sweep using a3 as the middle point.
+         */
+
+        // clockwise 1
+        if(FP_LT(r2, 0) && (FP_GT(r3, 0) || FP_LT(r3, r2)))
         {
-                sweep = a3 - a1;
+            sweep = (FP_GT(r3, 0)) ? (r3 - 2 * M_PI) : r3;
         }
-        /* Counter-clockwise */
-        else if(a1 < a2 && a2 < a3)
+        // clockwise 2
+        else if(FP_GT(r2, 0) && FP_GT(r3, 0) && FP_LT(r3, r2))
         {
-                sweep = a3 - a1;
+            sweep = (FP_GT(r3, 0)) ? (r3 - 2 * M_PI) : r3;
         }
-        /* Clockwise, wrap */
-        else if((a1 < a2 && a1 > a3) || (a2 < a3 && a1 > a3))
+        // counter-clockwise 4
+        else if(FP_GT(r2, 0) && (FP_LT(r3, 0) || FP_GT(r3, r2)))
         {
-                sweep = a3 - a1 + 2*M_PI;
+            sweep = (FP_LT(r3, 0)) ? (r3 + 2 * M_PI) : r3;
         }
-        /* Counter-clockwise, wrap */
-        else if((a1 > a2 && a1 < a3) || (a2 > a3 && a1 < a3))
+        // counter-clockwisk 5
+        else if(FP_LT(r2, 0) && FP_LT(r3, 0) && FP_GT(r3, r2))
         {
-                sweep = a3 - a1 - 2*M_PI;
-        } 
-        else 
+            sweep = (FP_LT(r3, 0)) ? (r3 + 2 * M_PI) : r3;
+        }
+        // clockwise invalid 3
+        else if(FP_GT(r2, 0) && (FP_GT(r3, r2) || FP_LT(r3, 0)))
         {
-                sweep = 0.0;
+            sweep = (FP_GT(r2, 0)) ? (r2 - 2 * M_PI) : r2;
+        }
+        // clockwise invalid 6
+        else
+        {
+            sweep = (FP_LT(r2, 0)) ? (r2 + 2 * M_PI) : r2;
         }
 
 #ifdef PGIS_DEBUG
@@ -340,50 +363,77 @@ lwcircle_compute_box3d(POINT4D *p1, POINT4D *p2, POINT4D *p3)
         for(i=0; i < 6; i++)
         {
                 switch(i) {
+		// right extent
                 case 0:
                         angle = 0.0;
                         xe = center->x + radius;
                         ye = center->y;
                         break;
+		// top extent
                 case 1:
                         angle = M_PI_2;
                         xe = center->x;
                         ye = center->y + radius;
                         break;
+		// left extent
                 case 2:
                         angle = M_PI;
                         xe = center->x - radius;
                         ye = center->y;
                         break;
+		// bottom extent
                 case 3:
                         angle = -1 * M_PI_2;
                         xe = center->x;
                         ye = center->y - radius;
                         break;
+		// first point
                 case 4:
                         angle = a1;
                         xe = p1->x;
                         ye = p1->y;
                         break;
+		// last point
                 case 5:
                         angle = a3;
                         xe = p3->x;
                         ye = p3->y;
                         break;
                 }
+		// determine if the extents are outside the arc
                 if(i < 4) 
                 {
-                        if(sweep > 0.0 && (angle > a3 || angle < a1)) continue;
-                        if(sweep < 0.0 && (angle < a3 || angle > a1)) continue;
+                    if(FP_GT(sweep, 0.0))
+                    {
+			if(FP_LT(a3, a1))
+			{
+			    if(FP_GT(angle, (a3 + 2 * M_PI)) || FP_LT(angle, a1)) continue;
+			}
+			else
+			{
+			    if(FP_GT(angle, a3) || FP_LT(angle, a1)) continue;
+			}
+                    }
+                    else
+                    {
+			if(FP_GT(a3, a1))
+			{
+ 			    if(FP_LT(angle, (a3 - 2 * M_PI)) || FP_GT(angle, a1)) continue;
+			}
+			else
+			{
+			   if(FP_LT(angle, a3) || FP_GT(angle, a1)) continue;
+			}
+                    }
                 }
 
 #ifdef PGIS_DEBUG
                 lwnotice("lwcircle_compute_box3d: potential extreame %d (%.16f, %.16f)", i, xe, ye);
 #endif
-                x1 = (x1 < xe) ? x1 : xe;
-                y1 = (y1 < ye) ? y1 : ye;
-                x2 = (x2 > xe) ? x2 : xe;
-                y2 = (y2 > ye) ? y2 : ye;
+                x1 = (FP_LT(x1, xe)) ? x1 : xe;
+                y1 = (FP_LT(y1, ye)) ? y1 : ye;
+                x2 = (FP_GT(x2, xe)) ? x2 : xe;
+                y2 = (FP_GT(y2, ye)) ? y2 : ye;
         }
 #ifdef PGIS_DEBUG
         lwnotice("lwcircle_compute_box3d: extreames found (%.16f %.16f, %.16f %.16f)", x1, y1, x2, y2);
@@ -395,10 +445,10 @@ lwcircle_compute_box3d(POINT4D *p1, POINT4D *p2, POINT4D *p3)
         y1 = center->y + y1 * radius;
         y2 = center->y + y2 * radius;
         */
-        z1 = (p1->z < p2->z) ? p1->z : p2->z;
-        z1 = (z1 < p3->z) ? z1 : p3->z;
-        z2 = (p1->z > p2->z) ? p1->z : p2->z;
-        z2 = (z2 > p3->z) ? z2 : p3->z;
+        z1 = (FP_LT(p1->z, p2->z)) ? p1->z : p2->z;
+        z1 = (FP_LT(z1, p3->z)) ? z1 : p3->z;
+        z2 = (FP_GT(p1->z, p2->z)) ? p1->z : p2->z;
+        z2 = (FP_GT(z2, p3->z)) ? z2 : p3->z;
 
         box = lwalloc(sizeof(BOX3D));
         box->xmin = x1; box->xmax = x2;
