@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: lwgeom.c 2687 2007-08-13 07:06:58Z mcayland $
+ * $Id: lwgeom.c 3272 2008-11-07 13:18:11Z mcayland $
  *
  * PostGIS - Spatial Types for PostgreSQL
  * http://postgis.refractions.net
@@ -711,6 +711,7 @@ lwgeom_longitude_shift(LWGEOM *lwgeom)
 			for (i=0; i<poly->nrings; i++)
 				ptarray_longitude_shift(poly->rings[i]);
 			return;
+		case MULTIPOINTTYPE:
 		case MULTILINETYPE:
 		case MULTIPOLYGONTYPE:
 		case COLLECTIONTYPE:
@@ -719,8 +720,8 @@ lwgeom_longitude_shift(LWGEOM *lwgeom)
 				lwgeom_longitude_shift(coll->geoms[i]);
 			return;
 		default:
-			lwerror("%s:%d: unknown geom type: %d",
+			lwerror("%s:%d: unsupported geom type: %s",
 				__FILE__, __LINE__,
-				TYPE_GETTYPE(lwgeom->type));
+				lwgeom_typename(TYPE_GETTYPE(lwgeom->type)));
 	}
 }
