@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: lwgeom_estimate.c 2761 2008-04-10 06:40:33Z mcayland $
+ * $Id: lwgeom_estimate.c 3334 2008-11-25 10:06:58Z mcayland $
  *
  * PostGIS - Spatial Types for PostgreSQL
  * http://postgis.refractions.net
@@ -485,7 +485,7 @@ Datum build_lwhistogram2d(PG_FUNCTION_ARGS)
 					 */
 
 #if DEBUG_GEOMETRY_STATS
-	elog(NOTICE,"box is : (%.15g,%.15g to %.15g,%.15g)",box->low.x,box->low.y, box->high.x, box->high.y);
+	elog(NOTICE,"box is : (%.15g,%.15g to %.15g,%.15g)",box->xmin,box->ymin, box->xmax, box->ymax);
 	elog(NOTICE,"        search is in x: %i to %i   y: %i to %i",x_idx_min, x_idx_max, y_idx_min,y_idx_max);
 #endif
 
@@ -698,7 +698,7 @@ Datum estimate_lwhistogram2d(PG_FUNCTION_ARGS)
 
 #if DEBUG_GEOMETRY_STATS
 elog(NOTICE,"start estimate_histogram2d: ");
-elog(NOTICE,"box is : (%.15g,%.15g to %.15g,%.15g)",box->low.x,box->low.y, box->high.x, box->high.y);
+elog(NOTICE,"box is : (%.15g,%.15g to %.15g,%.15g)",box->xmin,box->ymin, box->xmax, box->ymax);
 #endif
 
 	box_area = (box->xmax-box->xmin)*(box->ymax-box->ymin);
@@ -1383,6 +1383,7 @@ Datum LWGEOM_estimated_extent(PG_FUNCTION_ARGS)
 #if DEBUG_GEOMETRY_STATS
 		elog(NOTICE, " %d stat rows", SPI_processed);
 #endif
+		elog(ERROR, "LWGEOM_estimated_extent: couldn't locate table within current schema");
 		PG_RETURN_NULL() ;
 	}
 
@@ -1396,6 +1397,7 @@ Datum LWGEOM_estimated_extent(PG_FUNCTION_ARGS)
 #if DEBUG_GEOMETRY_STATS
 		elog(NOTICE, " stats are NULL");
 #endif
+		elog(ERROR, "LWGEOM_estimated_extent: couldn't locate statistics for table");
 		PG_RETURN_NULL();
 	}
 
@@ -2104,8 +2106,8 @@ compute_geometry_stats(VacAttrStats *stats, AnalyzeAttrFetchFunc fetchfunc,
 			continue;
 		}
 		if ( ! newhistobox ) {
-			newhistobox = palloc(sizeof(BOX));
-			memcpy(newhistobox, box, sizeof(BOX));
+			newhistobox = palloc(sizeof(BOX2DFLOAT4));
+			memcpy(newhistobox, box, sizeof(BOX2DFLOAT4));
 		} else {
 			if ( box->xmin < newhistobox->xmin )
 				newhistobox->xmin = box->xmin;
@@ -2574,6 +2576,7 @@ Datum LWGEOM_estimated_extent(PG_FUNCTION_ARGS)
 #if DEBUG_GEOMETRY_STATS
 		elog(NOTICE, " %d stat rows", SPI_processed);
 #endif
+		elog(ERROR, "LWGEOM_estimated_extent: couldn't locate table within current schema");
 		PG_RETURN_NULL() ;
 	}
 
@@ -2587,6 +2590,7 @@ Datum LWGEOM_estimated_extent(PG_FUNCTION_ARGS)
 #if DEBUG_GEOMETRY_STATS
 		elog(NOTICE, " stats are NULL");
 #endif
+		elog(ERROR, "LWGEOM_estimated_extent: couldn't locate statistics for table");
 		PG_RETURN_NULL();
 	}
 	if ( ArrayGetNItems(ARR_NDIM(array), ARR_DIMS(array)) != 4 )
