@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: lwgeom_functions_basic.c 2761 2008-04-10 06:40:33Z mcayland $
+ * $Id: lwgeom_functions_basic.c 3407 2008-12-15 10:35:03Z mcayland $
  *
  * PostGIS - Spatial Types for PostgreSQL
  * http://postgis.refractions.net
@@ -34,6 +34,7 @@ Datum LWGEOM_summary(PG_FUNCTION_ARGS);
 Datum LWGEOM_npoints(PG_FUNCTION_ARGS);
 Datum LWGEOM_nrings(PG_FUNCTION_ARGS);
 Datum LWGEOM_area_polygon(PG_FUNCTION_ARGS);
+Datum LWGEOM_dwithin(PG_FUNCTION_ARGS);
 Datum postgis_uses_stats(PG_FUNCTION_ARGS);
 Datum postgis_autocache_bbox(PG_FUNCTION_ARGS);
 Datum postgis_scripts_released(PG_FUNCTION_ARGS);
@@ -204,6 +205,7 @@ lwgeom_npoints(uchar *serialized)
 		LWLINE *line=NULL;
 		LWPOINT *point=NULL;
 		LWPOLY *poly=NULL;
+		LWCURVE *curve=NULL;
 		uchar *subgeom=NULL;
 
 		point = lwgeom_getpoint_inspected(inspected, i);
@@ -230,6 +232,13 @@ lwgeom_npoints(uchar *serialized)
 			continue;
 		}
 
+		curve = lwgeom_getcurve_inspected(inspected, i);
+		if (curve != NULL)
+		{
+			npoints += curve->points->npoints;
+			continue;
+		}
+
 		subgeom = lwgeom_getsubgeometry_inspected(inspected, i);
 		if ( subgeom != NULL )
 		{
@@ -1505,7 +1514,7 @@ Datum LWGEOM_force_collection(PG_FUNCTION_ARGS)
 	lwgeom = lwgeom_deserialize(SERIALIZED_FORM(geom));
 
 	/* alread a multi*, just make it a collection */
-	if ( TYPE_GETTYPE(lwgeom->type) >= MULTIPOINTTYPE )
+	if ( lwgeom_contains_subgeoms(TYPE_GETTYPE(lwgeom->type)) )
 	{
 		TYPE_SETTYPE(lwgeom->type, COLLECTIONTYPE);
 	}
@@ -1553,7 +1562,7 @@ Datum LWGEOM_force_multi(PG_FUNCTION_ARGS)
 	 * in input. If bbox cache is not there we'll need to handle
 	 * automatic bbox addition FOR_COMPLEX_GEOMS.
 	 */
-	if ( TYPE_GETTYPE(geom->type) >= MULTIPOINTTYPE &&
+	if ( lwgeom_contains_subgeoms(TYPE_GETTYPE(geom->type)) &&
 		TYPE_HASBBOX(geom->type) )
 	{
 		PG_RETURN_POINTER(geom);
@@ -1563,8 +1572,8 @@ Datum LWGEOM_force_multi(PG_FUNCTION_ARGS)
 	lwgeom = lwgeom_deserialize(SERIALIZED_FORM(geom));
 	type = TYPE_GETTYPE(lwgeom->type);
 
-	/* single geom, make it a multi */
-	if ( type < MULTIPOINTTYPE )
+	/* if it's a single POINT, LINESTRING or POLYGON geom, make it a multi */
+	if ( type == POINTTYPE || type == LINETYPE || type == POLYGONTYPE )
 	{
 		type += 3;
 		SRID = lwgeom->SRID;
@@ -1622,6 +1631,50 @@ Datum LWGEOM_mindistance2d(PG_FUNCTION_ARGS)
 	PG_RETURN_FLOAT8(mindist);
 }
 
+/* Minimum 2d distance between objects in geom1 and geom2. */
+PG_FUNCTION_INFO_V1(LWGEOM_dwithin);
+Datum LWGEOM_dwithin(PG_FUNCTION_ARGS)
+{
+	PG_LWGEOM *geom1;
+	PG_LWGEOM *geom2;
+	double mindist, tolerance;
+
+#ifdef PROFILE
+	profstart(PROF_QRUN);
+#endif
+
+	geom1 = (PG_LWGEOM *)PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
+	geom2 = (PG_LWGEOM *)PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
+  tolerance = PG_GETARG_FLOAT8(2);
+
+  if( tolerance < 0 ) {		
+    elog(ERROR,"Tolerance cannot be less than zero\n");
+		PG_RETURN_NULL();
+  }
+
+	if (pglwgeom_getSRID(geom1) != pglwgeom_getSRID(geom2))
+	{
+		elog(ERROR,"Operation on two GEOMETRIES with different SRIDs\n");
+		PG_RETURN_NULL();
+	}
+
+	mindist = lwgeom_mindistance2d_recursive_tolerance(
+	            SERIALIZED_FORM(geom1),
+		          SERIALIZED_FORM(geom2), 
+		          tolerance
+		        );
+
+#ifdef PROFILE
+	profstop(PROF_QRUN);
+	profreport("dist",geom1, geom2, NULL);
+#endif
+
+	PG_FREE_IF_COPY(geom1, 0);
+	PG_FREE_IF_COPY(geom2, 1);
+
+	PG_RETURN_BOOL(tolerance >= mindist);
+}
+
 /*
  *  Maximum 2d distance between linestrings.
  *  Returns NULL if given geoms are not linestrings.
@@ -3272,6 +3325,7 @@ lwgeom_affine_recursive(uchar *serialized,
 		LWLINE *line=NULL;
 		LWPOINT *point=NULL;
 		LWPOLY *poly=NULL;
+		LWCURVE *curve=NULL;
 		uchar *subgeom=NULL;
 
 		point = lwgeom_getpoint_inspected(inspected, i);
@@ -3313,6 +3367,18 @@ lwgeom_affine_recursive(uchar *serialized,
 			continue;
 		}
 
+		curve = lwgeom_getcurve_inspected(inspected, i);
+		if (curve != NULL)
+		{
+			lwgeom_affine_ptarray(curve->points,
+				afac, bfac, cfac,
+				dfac, efac, ffac,
+				gfac, hfac, ifac,
+				xoff, yoff, zoff);
+			lwgeom_release((LWGEOM *)curve);
+			continue;
+		}
+
 		subgeom = lwgeom_getsubgeometry_inspected(inspected, i);
 		if ( subgeom == NULL )
 		{
