@@ -593,6 +593,20 @@ Datum LWGEOM_gist_consistent(PG_FUNCTION_ARGS)
 	StrategyNumber strategy = (StrategyNumber) PG_GETARG_UINT16(2);
 	bool result;
 	BOX2DFLOAT4  box;
+#if USE_VERSION > 72
+	/* For sliced de-toasting code */
+	uchar *serialized_lwgeom;
+#endif
+
+#if USE_VERSION >= 84
+	/* PostgreSQL 8.4 and later require the RECHECK flag to be set here,
+  	   rather than being supplied as part of the operator class definition */
+	bool *recheck = (bool *) PG_GETARG_POINTER(4);
+
+	/* And since the index is lossy from conversion from float8 to float4, we must
+  	recheck */
+	*recheck = true;
+#endif
 
 #ifdef PGIS_DEBUG_CALLS
 	elog(NOTICE,"GIST: LWGEOM_gist_consistent called");
@@ -604,7 +618,29 @@ Datum LWGEOM_gist_consistent(PG_FUNCTION_ARGS)
 		PG_RETURN_BOOL(false); /* null query - this is screwy! */
 	}
 
-	query = (PG_LWGEOM*)PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
+#if USE_VERSION <= 72
+	/* 
+	** PgSQL 7.2 doesn't support sliced detoasting.
+	*/
+	query = (PG_LWGEOM*)PG_DETOAST_DATUM(PG_GETARG_DATUM(1)); 
+
+	if ( ! (DatumGetPointer(entry->key) != NULL && query) )
+	{
+		PG_FREE_IF_COPY(query, 1);
+		elog(ERROR, "LWGEOM_gist_consistent got either NULL query or entry->key");
+		PG_RETURN_BOOL(FALSE);
+	}
+	if ( ! getbox2d_p(SERIALIZED_FORM(query), &box) )
+	{
+		PG_FREE_IF_COPY(query, 1);
+		PG_RETURN_BOOL(FALSE);
+	}
+#else
+	/* 
+	** First pull only a small amount of the tuple, enough to 
+	** get the bounding box, if one exists.
+	*/
+	query = (PG_LWGEOM*)PG_DETOAST_DATUM_SLICE(PG_GETARG_DATUM(1), 0, VARHDRSZ + 1 + sizeof(BOX2DFLOAT4) );
 
 	if ( ! (DatumGetPointer(entry->key) != NULL && query) )
 	{
@@ -613,11 +649,26 @@ Datum LWGEOM_gist_consistent(PG_FUNCTION_ARGS)
 		PG_RETURN_BOOL(FALSE);
 	}
 
-	if ( ! getbox2d_p(SERIALIZED_FORM(query), &box) )
+	/*
+	** If the bounding box exists, copy it into the working variable.
+	** If not, pull the full toasted data out, and call the standard box 
+	** retrieval function, which will calculate the box from scratch.
+	*/
+	serialized_lwgeom = SERIALIZED_FORM(query);
+	if( lwgeom_hasBBOX(serialized_lwgeom[0]) ) 
 	{
-		PG_FREE_IF_COPY(query, 1);
-		PG_RETURN_BOOL(FALSE);
+		memcpy(&box, serialized_lwgeom + 1, sizeof(BOX2DFLOAT4));
+	} 
+	else 
+	{
+		query = (PG_LWGEOM*)PG_DETOAST_DATUM(PG_GETARG_DATUM(1)); 
+		if ( ! getbox2d_p(SERIALIZED_FORM(query), &box) )
+		{
+			PG_FREE_IF_COPY(query, 1);
+			PG_RETURN_BOOL(FALSE);
+		}
 	}
+#endif
 
 	if (GIST_LEAF(entry))
 		result = lwgeom_rtree_leaf_consistent((BOX2DFLOAT4 *)
