@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: lwgeom_gml.c 2710 2007-10-10 21:27:51Z mcayland $
+ * $Id: lwgeom_gml.c 3256 2008-11-05 11:27:37Z mcayland $
  *
  * PostGIS - Spatial Types for PostgreSQL
  * http://postgis.refractions.net
@@ -150,9 +150,16 @@ geometry_to_gml2(uchar *geom, char *srs)
 			poly = lwpoly_deserialize(geom);
 			return asgml2_poly(poly, srs);
 
-		default:
+		case MULTIPOINTTYPE:
+		case MULTILINETYPE:
+		case MULTIPOLYGONTYPE:	
+		case COLLECTIONTYPE:
 			inspected = lwgeom_inspect(geom);
 			return asgml2_inspected(inspected, srs);
+
+		default:
+                        lwerror("geometry_to_gml2: '%s' geometry type not supported", lwgeom_typename(type));
+                        return NULL;
 	}
 }
 
