@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: lwgeom_transform.c 2754 2008-03-31 11:16:54Z mcayland $
+ * $Id: lwgeom_transform.c 3274 2008-11-07 13:54:21Z mcayland $
  *
  * PostGIS - Spatial Types for PostgreSQL
  * http://postgis.refractions.net
@@ -750,6 +750,7 @@ lwgeom_transform_recursive(uchar *geom, PJ *inpj, PJ *outpj)
 		LWLINE *line=NULL;
 		LWPOINT *point=NULL;
 		LWPOLY *poly=NULL;
+		LWCURVE *curve=NULL;
 		POINT4D p;
 		uchar *subgeom=NULL;
 
@@ -795,6 +796,20 @@ lwgeom_transform_recursive(uchar *geom, PJ *inpj, PJ *outpj)
 			continue;
 		}
 
+		curve = lwgeom_getcurve_inspected(inspected, j);
+		if (curve != NULL)
+		{
+			POINTARRAY *pts = curve->points;
+			for (i=0; i<pts->npoints; i++)
+			{
+				getPoint4d_p(pts, i, &p);
+				transform_point(&p, inpj, outpj);
+				setPoint4d(pts, i, &p);
+			}
+			lwgeom_release((LWGEOM *)curve);
+			continue;
+		}
+
 		subgeom = lwgeom_getsubgeometry_inspected(inspected, j);
 		if ( subgeom != NULL )
 		{
