@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: ptarray.c 2612 2007-06-01 09:54:00Z mcayland $
+ * $Id: ptarray.c 3358 2008-12-03 16:27:44Z mcayland $
  *
  * PostGIS - Spatial Types for PostgreSQL
  * http://postgis.refractions.net
@@ -852,7 +852,7 @@ dynptarray_addPoint4d(DYNPTARRAY *dpa, POINT4D *p4d, int allow_duplicates)
 		 * return 0 and do nothing else if previous point in list is
 		 * equal to this one  (4D equality)
 		 */
-		if ( ! memcmp(p4d, &tmp, sizeof(POINT4D)) ) return 0;
+		if (tmp.x == p4d->x && tmp.y == p4d->y && tmp.z == p4d->z && tmp.m == p4d->m) return 0;
 	}
 
 	++pa->npoints;
